@@ -3,6 +3,7 @@
 
 import io
 import json
+import torch
 from tempfile import NamedTemporaryFile
 
 import joblib
@@ -375,7 +376,7 @@ class plMLP(pl.LightningModule):
         self.layers = nn.Sequential(*layers)
         if setting == "classification":
             self.layers.append(nn.Sigmoid())
-            self.loss = nn.CrossEntropyLoss()
+            self.loss = nn.BCEWithLogitsLoss()
         else:
             self.loss = nn.MSELoss()
 
@@ -386,7 +387,8 @@ class plMLP(pl.LightningModule):
         x, y = batch
         x = x.view(x.size(0), -1)
         y_hat = self.layers(x)
-        loss = self.loss(y_hat, y)
+        loss = self.loss(y_hat.flatten(), y.float())
+       
         self.log("train_loss", loss)
         return loss
 
@@ -522,6 +524,25 @@ class TorchMLP(BaseModel):
 
         self.network = torch.load(io.BytesIO(d["save"]))
 
+class XGBRegressor(BaseEstimator):
+
+    """Wrapper for xgboost XGBRegressor"""
+
+    @log_arguments
+    def __init__(self, *args, **kwargs):
+        from xgboost import XGBRegressor
+
+        self.obj = XGBRegressor(*args, **kwargs)
+
+class XGBClassifier(BaseEstimator):
+
+    """Wrapper for xgboost XGBRegressor"""
+
+    @log_arguments
+    def __init__(self, *args, **kwargs):
+        from xgboost import XGBClassifier
+
+        self.obj = XGBClassifier(*args, **kwargs)
 
 class XGBoostModel(BaseSKLearnModel, BaseObject):
     """XGBoost model
@@ -555,7 +576,6 @@ class XGBoostModel(BaseSKLearnModel, BaseObject):
         colsample_bytree=0.8,
         **kwargs
     ):
-        from xgboost import XGBClassifier, XGBRegressor
         import torch
 
         regressor = XGBRegressor(
