@@ -207,44 +207,45 @@
 #   3. Added condition to sanitize SMILES
 
 import numpy as np
-from rdkit import Chem
-from rdkit.Chem import AllChem
-from rdkit.DataStructs.cDataStructs import TanimotoSimilarity
-from rdkit.Chem import Mol
+from rdkit import Chem, RDLogger
+from rdkit.Chem import AllChem, Mol
 from rdkit.Chem.AtomPairs.Sheridan import GetBPFingerprint, GetBTFingerprint
 from rdkit.Chem.Pharm2D import Generate, Gobbi_Pharm2D
-
-from rdkit import RDLogger
+from rdkit.DataStructs.cDataStructs import TanimotoSimilarity
 
 import olorenchemengine as oce
 
 RDLogger.DisableLog("rdApp.*")
 
+
 def randomize_smiles(mol):
-    '''Returns a random (dearomatized) SMILES given an rdkit mol object of a molecule.
+    """Returns a random (dearomatized) SMILES given an rdkit mol object of a molecule.
     Parameters:
     mol (rdkit.Chem.rdchem.Mol) :  RdKit mol object (None if invalid smile string smi)
-    
+
     Returns:
     mol (rdkit.Chem.rdchem.Mol) : RdKit mol object  (None if invalid smile string smi)
-    '''
+    """
     if not mol:
         return None
 
     Chem.Kekulize(mol)
-    return Chem.MolToSmiles(mol, canonical=False, doRandom=True, isomericSmiles=False,  kekuleSmiles=True)
+    return Chem.MolToSmiles(
+        mol, canonical=False, doRandom=True, isomericSmiles=False, kekuleSmiles=True
+    )
+
 
 def sanitize_smiles(smi):
-    '''Return a canonical smile representation of smi
-    
+    """Return a canonical smile representation of smi
+
     Parameters:
-    smi (string) : smile string to be canonicalized 
-    
+    smi (string) : smile string to be canonicalized
+
     Returns:
     mol (rdkit.Chem.rdchem.Mol) : RdKit mol object                          (None if invalid smile string smi)
     smi_canon (string)          : Canonicalized smile representation of smi (None if invalid smile string smi)
-    conversion_successful (bool): True/False to indicate if conversion was  successful 
-    '''
+    conversion_successful (bool): True/False to indicate if conversion was  successful
+    """
     try:
         mol = Chem.MolFromSmiles(smi, sanitize=True)
         smi_canon = Chem.MolToSmiles(mol, isomericSmiles=False, canonical=True)
@@ -253,138 +254,158 @@ def sanitize_smiles(smi):
         return (mol, smi_canon, True)
     except:
         return (None, None, False)
-    
+
 
 def get_selfie_chars(selfie):
-    '''Obtain a list of all selfie characters in string selfie
-    
-    Parameters: 
-    selfie (string) : A selfie string - representing a molecule 
-    
-    Example: 
+    """Obtain a list of all selfie characters in string selfie
+
+    Parameters:
+    selfie (string) : A selfie string - representing a molecule
+
+    Example:
     >>> get_selfie_chars('[C][=C][C][=C][C][=C][Ring1][Branch1_1]')
     ['[C]', '[=C]', '[C]', '[=C]', '[C]', '[=C]', '[Ring1]', '[Branch1_1]']
-    
+
     Returns:
     chars_selfie: list of selfie characters present in molecule selfie
-    '''
-    chars_selfie = [] # A list of all SELFIE sybols from string selfie
-    while selfie != '':
-        chars_selfie.append(selfie[selfie.find('['): selfie.find(']')+1])
-        selfie = selfie[selfie.find(']')+1:]
+    """
+    chars_selfie = []  # A list of all SELFIE sybols from string selfie
+    while selfie != "":
+        chars_selfie.append(selfie[selfie.find("[") : selfie.find("]") + 1])
+        selfie = selfie[selfie.find("]") + 1 :]
     return chars_selfie
 
+
 def mutate_selfie(selfie, max_molecules_len, write_fail_cases=False, index=None):
-    '''Return a mutated selfie string (only one mutation on selfie is performed)
-    
-    Mutations are done until a valid molecule is obtained 
-    Rules of mutation: With a 33.3% propbabily, either: 
+    """Return a mutated selfie string (only one mutation on selfie is performed)
+
+    Mutations are done until a valid molecule is obtained
+    Rules of mutation: With a 33.3% propbabily, either:
         1. Add a random SELFIE character in the string
         2. Replace a random SELFIE character with another
         3. Delete a random character
-    
+
     Parameters:
-    selfie            (string)  : SELFIE string to be mutated 
+    selfie            (string)  : SELFIE string to be mutated
     max_molecules_len (int)     : Mutations of SELFIE string are allowed up to this length
     write_fail_cases  (bool)    : If true, failed mutations are recorded in "selfie_failure_cases.txt"
-    
+
     Returns:
     selfie_mutated    (string)  : Mutated SELFIE string
     smiles_canon      (string)  : canonical smile of mutated SELFIE string
-    '''
-    valid=False
+    """
+    valid = False
     fail_counter = 0
     chars_selfie = get_selfie_chars(selfie)
-    
+
     while not valid:
         fail_counter += 1
-        
+
         oce.import_or_install("selfies")
-        
+
         import selfies
 
-        alphabet = list(selfies.get_semantic_robust_alphabet()) # 34 SELFIE characters 
+        alphabet = list(selfies.get_semantic_robust_alphabet())  # 34 SELFIE characters
 
-        choice_ls = [1, 2, 3] # 1=Insert; 2=Replace; 3=Delete
+        choice_ls = [1, 2, 3]  # 1=Insert; 2=Replace; 3=Delete
         random_choice = np.random.choice(choice_ls, 1)[0]
-        
+
         # Insert a character in a Random Location
-        if random_choice == 1: 
+        if random_choice == 1:
             if index is None:
-                random_index = np.random.randint(len(chars_selfie)+1)
+                random_index = np.random.randint(len(chars_selfie) + 1)
             else:
                 random_index = index
             random_character = np.random.choice(alphabet, size=1)[0]
-            
-            selfie_mutated_chars = chars_selfie[:random_index] + [random_character] + chars_selfie[random_index:]
 
-        # Replace a random character 
+            selfie_mutated_chars = (
+                chars_selfie[:random_index]
+                + [random_character]
+                + chars_selfie[random_index:]
+            )
+
+        # Replace a random character
         elif random_choice == 2:
-            if index is None:                         
+            if index is None:
                 random_index = np.random.randint(len(chars_selfie))
             else:
                 random_index = index
             random_character = np.random.choice(alphabet, size=1)[0]
             if random_index == 0:
-                selfie_mutated_chars = [random_character] + chars_selfie[random_index+1:]
+                selfie_mutated_chars = [random_character] + chars_selfie[
+                    random_index + 1 :
+                ]
             else:
-                selfie_mutated_chars = chars_selfie[:random_index] + [random_character] + chars_selfie[random_index+1:]
-                
+                selfie_mutated_chars = (
+                    chars_selfie[:random_index]
+                    + [random_character]
+                    + chars_selfie[random_index + 1 :]
+                )
+
         # Delete a random character
         elif random_choice == 3:
-            if index is None: 
+            if index is None:
                 random_index = np.random.randint(len(chars_selfie))
             else:
                 random_index = index
             if random_index == 0:
-                selfie_mutated_chars = chars_selfie[random_index+1:]
+                selfie_mutated_chars = chars_selfie[random_index + 1 :]
             else:
-                selfie_mutated_chars = chars_selfie[:random_index] + chars_selfie[random_index+1:]
-                
-        else: 
-            raise Exception('Invalid Operation trying to be performed')
+                selfie_mutated_chars = (
+                    chars_selfie[:random_index] + chars_selfie[random_index + 1 :]
+                )
+
+        else:
+            raise Exception("Invalid Operation trying to be performed")
 
         selfie_mutated = "".join(x for x in selfie_mutated_chars)
         sf = "".join(x for x in chars_selfie)
-        
+
         try:
             smiles = decoder(selfie_mutated)
             mol, smiles_canon, done = sanitize_smiles(smiles)
-            if len(selfie_mutated_chars) > max_molecules_len or smiles_canon=="":
+            if len(selfie_mutated_chars) > max_molecules_len or smiles_canon == "":
                 done = False
             if done:
                 valid = True
             else:
                 valid = False
         except:
-            valid=False
+            valid = False
             if fail_counter > 1 and write_fail_cases == True:
                 f = open("selfie_failure_cases.txt", "a+")
-                f.write('Tried to mutate SELFIE: '+str(sf)+' To Obtain: '+str(selfie_mutated) + '\n')
+                f.write(
+                    "Tried to mutate SELFIE: "
+                    + str(sf)
+                    + " To Obtain: "
+                    + str(selfie_mutated)
+                    + "\n"
+                )
                 f.close()
-    
+
     return (selfie_mutated, smiles_canon)
 
-def get_mutated_SELFIES(selfies_ls, num_mutations): 
-    ''' Mutate all the SELFIES in 'selfies_ls' 'num_mutations' number of times. 
-    
+
+def get_mutated_SELFIES(selfies_ls, num_mutations):
+    """Mutate all the SELFIES in 'selfies_ls' 'num_mutations' number of times.
+
     Parameters:
-    selfies_ls   (list)  : A list of SELFIES 
+    selfies_ls   (list)  : A list of SELFIES
     num_mutations (int)  : number of mutations to perform on each SELFIES within 'selfies_ls'
-    
+
     Returns:
     selfies_ls   (list)  : A list of mutated SELFIES
-    
-    '''
-    for _ in range(num_mutations): 
+
+    """
+    for _ in range(num_mutations):
         selfie_ls_mut_ls = []
-        for str_ in selfies_ls: 
-            
+        for str_ in selfies_ls:
+
             str_chars = get_selfie_chars(str_)
             max_molecules_len = len(str_chars) + num_mutations
-            
+
             selfie_mutated, _ = mutate_selfie(str_, max_molecules_len)
             selfie_ls_mut_ls.append(selfie_mutated)
-        
+
         selfies_ls = selfie_ls_mut_ls.copy()
     return selfies_ls
