@@ -3,4 +3,4 @@ from .model_comparisons import *
 from .matched_pairs import *
 from .exploratory_analysis import *
 from .visualize_interpret import *
-from .util import sample_colorscale
+from .compounds import *
