@@ -59,6 +59,7 @@ class BaseVisualization(BaseClass):
         "d3": "https://d3js.org/d3.v4.js",
         "plotly": "https://cdn.plot.ly/plotly-2.14.0.min.js",
         "olorenrenderer": "https://unpkg.com/olorenrenderer@1.0.0-c/dist/oloren-renderer.min.js",
+        "smilesdrawer": "https://unpkg.com/smiles-drawer@1.0.10/dist/smiles-drawer.min.js",
         "rdkit": "https://unpkg.com/@rdkit/rdkit/dist/RDKit_minimal.js",
     }
 
@@ -121,6 +122,8 @@ class BaseVisualization(BaseClass):
         <!DOCTYPE html>
         <head>
         <meta charset="utf-8">
+        <meta http-equiv="x-ua-compatible" content="ie=edge" />
+        <meta name="viewport" content="width=device-width, initial-scale=1" />
         </head>
 
         <html style="height: 100%;">
@@ -280,7 +283,7 @@ class VisualizeError(BaseVisualization):
         self,
         dataset: Union[BaseDataset, list, pd.Series, np.ndarray],
         value: Union[int, float, np.ndarray],
-        error: Union[int, float, np.ndarray],
+        error: Union[tuple, list, np.ndarray],
         ci=None,
         box=False,
         points=True,
@@ -291,7 +294,7 @@ class VisualizeError(BaseVisualization):
         height=600,
         **kwargs,
     ):
-        assert error >= 0, "error must be nonnegative"
+        assert error[1] >= error[0], "Error tuple must be increasing."
 
         if box:
             self.box = box
@@ -311,10 +314,8 @@ class VisualizeError(BaseVisualization):
             assert len(value) == 1, "value length must be 1"
             value = value[0]
         self.value = value
-        if isinstance(error, np.ndarray):
-            assert len(error) == 1, "error length must be 1"
-            error = error[0]
-        self.error = error
+        self.lower_error = error[0]
+        self.upper_error = error[1]
 
         if xaxis_title is None:
             if hasattr(dataset, "property_col"):
@@ -340,7 +341,8 @@ class VisualizeError(BaseVisualization):
         d = {
             "reference": self.reference,
             "value": self.value,
-            "error": self.error,
+            "lower_error": self.lower_error,
+            "upper_error": self.upper_error,
             "title": self.title,
             "xaxis_title": self.xaxis_title,
             "yaxis_title": self.yaxis_title,
@@ -630,7 +632,7 @@ class CompoundScatterPlot(BaseVisualization):
         super().__init__(log=False, **kwargs)
 
         # Add packages to import for JavaScript
-        self.packages += ["plotly", "olorenrenderer"]
+        self.packages += ["plotly", "smilesdrawer"]
 
     @property
     def JS_NAME(self) -> str:
@@ -1079,7 +1081,7 @@ class MorganContributions(BaseVisualization):
         self.original_prediction, self.predictions = self._make_predictions(self.smiles)
         self.args = args
         self.kwargs = kwargs
-        self.packages = ["plotly", "rdkit", "olorenrenderer"]
+        self.packages = ["plotly", "rdkit", "smilesdrawer"]
 
     def _train_model(self):
         """Train random forest model based on the morgan vec representation"""
